package jcg.zheng.demo.constructor;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.junit.jupiter.api.Test;

public class CreateObjectViaReflectionTest {

	private Object createObjectWithNoArgsViaReflection(String className) {
		try {
			Class<?> c = Class.forName(className);
			Constructor<?> publicConstructors = c.getConstructor();
			return publicConstructors.newInstance();

		} catch (NoSuchMethodException | SecurityException | InstantiationException | IllegalAccessException
				| IllegalArgumentException | InvocationTargetException | ClassNotFoundException e) {

			return null;
		}
	}
	
	private Object createObjectWithStringArgsViaReflection(String className, String constructorArg) {
		try {
			Class<?> c = Class.forName(className);
			Constructor<?> publicConstructors = c.getConstructor(String.class);
			return publicConstructors.newInstance(new Object[] { constructorArg });

		} catch (NoSuchMethodException | SecurityException | InstantiationException | IllegalAccessException
				| IllegalArgumentException | InvocationTargetException | ClassNotFoundException e) {

			return null;
		}
	}

	@Test
	public void test_C1_P2_NoConstructor() {
		Object object = createObjectWithNoArgsViaReflection("jcg.zheng.demo.constructor.C1_P2_NoConstructor");
		assertNotNull(object);
	}

	@Test
	public void test_C2_P2_NoConstructor() {
		Object object = createObjectWithNoArgsViaReflection("jcg.zheng.demo.constructor.C2_P2_NoConstructor");
		assertNull(object);
		
		object = createObjectWithStringArgsViaReflection("jcg.zheng.demo.constructor.C2_P2_NoConstructor", "test");
		assertNotNull(object);
	}


	@Test
	public void test_C3_P1_HasDefaultConstructor() {
		Object object = createObjectWithNoArgsViaReflection("jcg.zheng.demo.constructor.C3_P1_HasDefaultConstructor");
		assertNotNull(object);
	}

	@Test
	public void test_C4_P3_NoDefaultConstructor() {
		Object object = createObjectWithNoArgsViaReflection("jcg.zheng.demo.constructor.C4_P3_NoDefaultConstructor");
		assertNull(object);
		
		object = createObjectWithStringArgsViaReflection("jcg.zheng.demo.constructor.C4_P3_NoDefaultConstructor", "test");
		assertNotNull(object);
	}

	@Test
	public void test_S5_Z_NoDefaultConstructor() {
		Object object = createObjectWithNoArgsViaReflection("jcg.zheng.demo.constructor.C5_P3_NoDefaultConstructor");
		assertNotNull(object);
	}

	@Test
	public void test_P1_HasDefaultConstructor() {
		Object object = createObjectWithNoArgsViaReflection("jcg.zheng.demo.constructor.P1_HasDefaultConstructor");
		assertNotNull(object);
	}

	@Test
	public void test_P2_NoConstructor() {
		Object object = createObjectWithNoArgsViaReflection("jcg.zheng.demo.constructor.P2_NoConstructor");
		assertNotNull(object);
	}

	@Test
	public void test_P3_NoDefaultConstructor() {
		Object object = createObjectWithNoArgsViaReflection("jcg.zheng.demo.constructor.P3_NoDefaultConstructor");
		assertNull(object);
		
		object = createObjectWithStringArgsViaReflection("jcg.zheng.demo.constructor.P3_NoDefaultConstructor", "test");
		assertNotNull(object);
	}
}
